@@ -92,7 +92,7 @@ var _ = Describe("Adaptation", func() {
 	When("SyncFn is nil", func() {
 		var (
 			syncFn   func(ctx context.Context, cb nri.SyncCB) error
-			updateFn = func(ctx context.Context, updates []*nri.ContainerUpdate) ([]*nri.ContainerUpdate, error) {
+			updateFn = func(_ context.Context, _ []*nri.ContainerUpdate) ([]*nri.ContainerUpdate, error) {
 				return nil, nil
 			}
 		)
@@ -119,7 +119,7 @@ var _ = Describe("Adaptation", func() {
 	When("UpdateFn is nil", func() {
 		var (
 			updateFn func(ctx context.Context, updates []*nri.ContainerUpdate) ([]*nri.ContainerUpdate, error)
-			syncFn   = func(ctx context.Context, cb nri.SyncCB) error {
+			syncFn   = func(_ context.Context, _ nri.SyncCB) error {
 				return nil
 			}
 		)
@@ -385,7 +385,7 @@ var _ = Describe("Pod and container requests and events", func() {
 					}
 
 					order       []*mockPlugin
-					recordOrder = func(p *mockPlugin, pod *api.PodSandbox, ctr *api.Container) error {
+					recordOrder = func(p *mockPlugin, _ *api.PodSandbox, _ *api.Container) error {
 						order = append(order, p)
 						return nil
 					}
@@ -435,7 +435,7 @@ var _ = Describe("Plugin container creation adjustments", func() {
 		s = &Suite{}
 	)
 
-	adjust := func(subject string, p *mockPlugin, pod *api.PodSandbox, ctr *api.Container, overwrite bool) (*api.ContainerAdjustment, []*api.ContainerUpdate, error) {
+	adjust := func(subject string, p *mockPlugin, _ *api.PodSandbox, _ *api.Container, overwrite bool) (*api.ContainerAdjustment, []*api.ContainerUpdate, error) {
 		plugin := p.idx + "-" + p.name
 		a := &api.ContainerAdjustment{}
 		switch subject {
@@ -455,6 +455,9 @@ var _ = Describe("Plugin container creation adjustments", func() {
 			}
 			a.AddMount(mnt)
 
+		case "remove mount":
+			a.RemoveMount("/remove/test/destination")
+
 		case "environment":
 			if overwrite {
 				a.RemoveEnv("key")
@@ -488,6 +491,13 @@ var _ = Describe("Plugin container creation adjustments", func() {
 		case "rlimit":
 			a.AddRlimit("nofile", 456, 123)
 
+		case "CDI-device":
+			a.AddCDIDevice(
+				&api.CDIDevice{
+					Name: "vendor0.com/dev=dev0",
+				},
+			)
+
 		case "resources/cpu":
 			a.SetLinuxCPUShares(123)
 			a.SetLinuxCPUQuota(456)
@@ -553,6 +563,13 @@ var _ = Describe("Plugin container creation adjustments", func() {
 						PodSandboxId: "pod0",
 						Name:         "ctr0",
 						State:        api.ContainerState_CONTAINER_CREATED, // XXX FIXME-kludge
+						Mounts: []*api.Mount{
+							{
+								Type:        "bind",
+								Source:      "/remove/test",
+								Destination: "/remove/test/destination",
+							},
+						},
 					}
 				)
 
@@ -565,12 +582,12 @@ var _ = Describe("Plugin container creation adjustments", func() {
 				s.Startup()
 
 				podReq := &api.RunPodSandboxRequest{Pod: pod}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq := &api.CreateContainerRequest{
 					Pod:       pod,
 					Container: ctr,
 				}
-				reply, err := runtime.runtime.CreateContainer(ctx, ctrReq)
+				reply, err := runtime.CreateContainer(ctx, ctrReq)
 				Expect(err).To(BeNil())
 				Expect(stripAdjustment(reply.Adjust)).Should(Equal(stripAdjustment(expected)))
 			},
@@ -592,6 +609,15 @@ var _ = Describe("Plugin container creation adjustments", func() {
 					},
 				},
 			),
+			Entry("remove a mount", "remove mount",
+				&api.ContainerAdjustment{
+					Mounts: []*api.Mount{
+						{
+							Destination: api.MarkForRemoval("/remove/test/destination"),
+						},
+					},
+				},
+			),
 			Entry("adjust environment", "environment",
 				&api.ContainerAdjustment{
 					Env: []*api.KeyValue{
@@ -632,6 +658,15 @@ var _ = Describe("Plugin container creation adjustments", func() {
 					Rlimits: []*api.POSIXRlimit{{Type: "nofile", Soft: 123, Hard: 456}},
 				},
 			),
+			Entry("adjust CDI Devices", "CDI-device",
+				&api.ContainerAdjustment{
+					CDIDevices: []*api.CDIDevice{
+						{
+							Name: "vendor0.com/dev=dev0",
+						},
+					},
+				},
+			),
 			Entry("adjust CPU resources", "resources/cpu",
 				&api.ContainerAdjustment{
 					Linux: &api.LinuxContainerAdjustment{
@@ -757,12 +792,12 @@ var _ = Describe("Plugin container creation adjustments", func() {
 				s.Startup()
 
 				podReq := &api.RunPodSandboxRequest{Pod: pod}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq := &api.CreateContainerRequest{
 					Pod:       pod,
 					Container: ctr,
 				}
-				reply, err := runtime.runtime.CreateContainer(ctx, ctrReq)
+				reply, err := runtime.CreateContainer(ctx, ctrReq)
 				if shouldFail {
 					Expect(err).ToNot(BeNil())
 				} else {
@@ -845,7 +880,7 @@ var _ = Describe("Plugin container updates during creation", func() {
 		s = &Suite{}
 	)
 
-	update := func(subject, which string, p *mockPlugin, pod *api.PodSandbox, ctr *api.Container) (*api.ContainerAdjustment, []*api.ContainerUpdate, error) {
+	update := func(subject, which string, p *mockPlugin, _ *api.PodSandbox, ctr *api.Container) (*api.ContainerAdjustment, []*api.ContainerUpdate, error) {
 		plugin := p.idx + "-" + p.name
 
 		if which != plugin && which != "*" && which != "both" {
@@ -948,21 +983,21 @@ var _ = Describe("Plugin container updates during creation", func() {
 				s.Startup()
 
 				podReq := &api.RunPodSandboxRequest{Pod: pod0}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq := &api.CreateContainerRequest{
 					Pod:       pod0,
 					Container: ctr0,
 				}
-				_, err := runtime.runtime.CreateContainer(ctx, ctrReq)
+				_, err := runtime.CreateContainer(ctx, ctrReq)
 				Expect(err).To(BeNil())
 
 				podReq = &api.RunPodSandboxRequest{Pod: pod1}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq = &api.CreateContainerRequest{
 					Pod:       pod1,
 					Container: ctr1,
 				}
-				reply, err = runtime.runtime.CreateContainer(ctx, ctrReq)
+				reply, err = runtime.CreateContainer(ctx, ctrReq)
 				Expect(err).To(BeNil())
 
 				Expect(len(reply.Update)).To(Equal(1))
@@ -1102,21 +1137,21 @@ var _ = Describe("Plugin container updates during creation", func() {
 				s.Startup()
 
 				podReq := &api.RunPodSandboxRequest{Pod: pod0}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq := &api.CreateContainerRequest{
 					Pod:       pod0,
 					Container: ctr0,
 				}
-				_, err := runtime.runtime.CreateContainer(ctx, ctrReq)
+				_, err := runtime.CreateContainer(ctx, ctrReq)
 				Expect(err).To(BeNil())
 
 				podReq = &api.RunPodSandboxRequest{Pod: pod1}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq = &api.CreateContainerRequest{
 					Pod:       pod1,
 					Container: ctr1,
 				}
-				reply, err = runtime.runtime.CreateContainer(ctx, ctrReq)
+				reply, err = runtime.CreateContainer(ctx, ctrReq)
 				if which == "both" {
 					Expect(err).ToNot(BeNil())
 				} else {
@@ -1216,7 +1251,7 @@ var _ = Describe("Solicited container updates by plugins", func() {
 		s = &Suite{}
 	)
 
-	update := func(subject, which string, p *mockPlugin, pod *api.PodSandbox, ctr *api.Container, r, exp *api.LinuxResources) ([]*api.ContainerUpdate, error) {
+	update := func(subject, which string, p *mockPlugin, _ *api.PodSandbox, ctr *api.Container, _, _ *api.LinuxResources) ([]*api.ContainerUpdate, error) {
 		plugin := p.idx + "-" + p.name
 
 		if which != plugin && which != "*" && which != "both" {
@@ -1306,12 +1341,12 @@ var _ = Describe("Solicited container updates by plugins", func() {
 				s.Startup()
 
 				podReq := &api.RunPodSandboxRequest{Pod: pod0}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq := &api.CreateContainerRequest{
 					Pod:       pod0,
 					Container: ctr0,
 				}
-				_, err := runtime.runtime.CreateContainer(ctx, ctrReq)
+				_, err := runtime.CreateContainer(ctx, ctrReq)
 				Expect(err).To(BeNil())
 
 				updReq := &api.UpdateContainerRequest{
@@ -1339,7 +1374,7 @@ var _ = Describe("Solicited container updates by plugins", func() {
 						},
 					},
 				}
-				reply, err = runtime.runtime.UpdateContainer(ctx, updReq)
+				reply, err = runtime.UpdateContainer(ctx, updReq)
 
 				Expect(len(reply.Update)).To(Equal(1))
 				Expect(err).To(BeNil())
@@ -1545,12 +1580,12 @@ var _ = Describe("Solicited container updates by plugins", func() {
 				s.Startup()
 
 				podReq := &api.RunPodSandboxRequest{Pod: pod0}
-				Expect(runtime.runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
+				Expect(runtime.RunPodSandbox(ctx, podReq)).To(Succeed())
 				ctrReq := &api.CreateContainerRequest{
 					Pod:       pod0,
 					Container: ctr0,
 				}
-				_, err := runtime.runtime.CreateContainer(ctx, ctrReq)
+				_, err := runtime.CreateContainer(ctx, ctrReq)
 				Expect(err).To(BeNil())
 
 				updReq := &api.UpdateContainerRequest{
@@ -1578,7 +1613,7 @@ var _ = Describe("Solicited container updates by plugins", func() {
 						},
 					},
 				}
-				reply, err = runtime.runtime.UpdateContainer(ctx, updReq)
+				reply, err = runtime.UpdateContainer(ctx, updReq)
 				if which == "both" {
 					Expect(err).ToNot(BeNil())
 				} else {
@@ -1807,7 +1842,7 @@ var _ = Describe("Unsolicited container update requests", func() {
 				recordedUpdates []*nri.ContainerUpdate
 			)
 
-			runtime.updateFn = func(ctx context.Context, updates []*nri.ContainerUpdate) ([]*nri.ContainerUpdate, error) {
+			runtime.updateFn = func(_ context.Context, updates []*nri.ContainerUpdate) ([]*nri.ContainerUpdate, error) {
 				recordedUpdates = updates
 				return nil, nil
 			}
@@ -1834,6 +1869,72 @@ var _ = Describe("Unsolicited container update requests", func() {
 	})
 })
 
+var _ = Describe("Plugin configuration request", func() {
+	var (
+		s = &Suite{}
+	)
+
+	AfterEach(func() {
+		s.Cleanup()
+	})
+
+	BeforeEach(func() {
+		s.Prepare(&mockRuntime{}, &mockPlugin{idx: "00", name: "test"})
+	})
+
+	It("should pass runtime version information to plugins", func() {
+		var (
+			runtimeName    = "test-runtime"
+			runtimeVersion = "1.2.3"
+		)
+
+		s.runtime.name = runtimeName
+		s.runtime.version = runtimeVersion
+
+		s.Startup()
+
+		Expect(s.plugins[0].RuntimeName()).To(Equal(runtimeName))
+		Expect(s.plugins[0].RuntimeVersion()).To(Equal(runtimeVersion))
+	})
+
+	When("unchanged", func() {
+		It("should pass default timeout information to plugins", func() {
+			var (
+				registerTimeout = nri.DefaultPluginRegistrationTimeout
+				requestTimeout  = nri.DefaultPluginRequestTimeout
+			)
+
+			s.Startup()
+			Expect(s.plugins[0].stub.RegistrationTimeout()).To(Equal(registerTimeout))
+			Expect(s.plugins[0].stub.RequestTimeout()).To(Equal(requestTimeout))
+		})
+	})
+
+	When("reconfigured", func() {
+		var (
+			registerTimeout = nri.DefaultPluginRegistrationTimeout + 5*time.Millisecond
+			requestTimeout  = nri.DefaultPluginRequestTimeout + 7*time.Millisecond
+		)
+
+		BeforeEach(func() {
+			nri.SetPluginRegistrationTimeout(registerTimeout)
+			nri.SetPluginRequestTimeout(requestTimeout)
+			s.Prepare(&mockRuntime{}, &mockPlugin{idx: "00", name: "test"})
+		})
+
+		AfterEach(func() {
+			nri.SetPluginRegistrationTimeout(nri.DefaultPluginRegistrationTimeout)
+			nri.SetPluginRequestTimeout(nri.DefaultPluginRequestTimeout)
+		})
+
+		It("should pass configured timeout information to plugins", func() {
+			s.Startup()
+			Expect(s.plugins[0].stub.RegistrationTimeout()).To(Equal(registerTimeout))
+			Expect(s.plugins[0].stub.RequestTimeout()).To(Equal(requestTimeout))
+		})
+	})
+})
+
 // Notes:
 //
 //	XXX FIXME KLUDGE
@@ -1858,6 +1959,7 @@ func stripAdjustment(a *api.ContainerAdjustment) *api.ContainerAdjustment {
 	stripHooks(a)
 	stripRlimits(a)
 	stripLinuxAdjustment(a)
+	stripCDIDevices(a)
 	return a
 }
 
@@ -1918,6 +2020,12 @@ func stripLinuxDevices(a *api.ContainerAdjustment) {
 	}
 }
 
+func stripCDIDevices(a *api.ContainerAdjustment) {
+	if len(a.CDIDevices) == 0 {
+		a.CDIDevices = nil
+	}
+}
+
 func stripLinuxResources(r *api.LinuxResources) *api.LinuxResources {
 	if r == nil {
 		return nil
